<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010 Ugolotti Federica
 * @license GNU/GPL, see LICENSE.php
 * Abook is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class AbookModelBooks extends JModel
{
	var $_data;
        var $_total = null;
        var $_pagination = null;
	var $_id = null;

        function __construct()
        {
                parent::__construct();

                global $mainframe, $option;

                // Get the pagination request variables
                $limit          = $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
                $limitstart     = $mainframe->getUserStateFromRequest( $option.JRequest::getCmd( 'controller').'.limitstart', 'limitstart', 0, 'int' );

                $this->setState('limit', $limit);
                $this->setState('limitstart', $limitstart);
		$array = JRequest::getVar('cid',  0, '', 'array');
                $this->setId((int)$array[0]);
		$row->catid      = JRequest::getInt('catid');
        }

	function setId($id)
        {
                // Set id and wipe data
                $this->_id       = $id;
                $this->_data = null;
        }

	function _buildQuery()
	{
                $where          = $this->_buildContentWhere();
                $orderby        = $this->_buildContentOrderBy();

		$query =  ' SELECT a.*, cc.title AS category, ee.name AS editor, ee.id AS editorid, '
			. ' GROUP_CONCAT(bb.name ORDER BY bb.name ASC) AS author, l.location AS location, lb.name AS library, g.name AS groupname, '
			. ' ROUND( v.rating_sum / v.rating_count ) AS rating, v.rating_count '
			. ' FROM #__abbook AS a '
			. ' LEFT JOIN #__abcategories AS cc ON cc.id = a.catid '
			. ' LEFT JOIN #__abeditor AS ee ON ee.id = a.ideditor '
			. ' LEFT JOIN #__abbookauth AS aa ON aa.idbook = a.id '
			. ' LEFT JOIN #__abauthor AS bb ON bb.id=aa.idauth '
			. ' LEFT JOIN #__users AS u ON u.id = a.checked_out '
			. ' LEFT JOIN #__ablocations AS l ON l.id = a.idlocation '
			. ' LEFT JOIN #__ablibrary AS lb ON lb.id = a.idlibrary '
			. ' LEFT JOIN #__groups AS g ON g.id = a.access '
			. ' LEFT JOIN #__abrating AS v ON a.id = v.book_id'
			. $where
			. ' GROUP BY a.id '
			. $orderby
		;
		return $query;
	}

	function getData()
	{
		// Lets load the data if it doesn't already exist
		if (empty( $this->_data ))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit')  );
		}

		return $this->_data;
	}

	function getauthor()
	{
		$row1 =& $this->getTable('bookauth2');
	}

        function getTotal()
        {
                // Lets load the total nr if it doesn't already exist
                if (empty($this->_total))
                {
			//vecchio metodo : se ci sono troppi dati la query impiega troppo tempo, meglio snellire la query
                        //$query = $this->_buildQuery();
                        //$this->_total = $this->_getListCount($query);
			//nuovo metodo: query più snella
			$where          = $this->_buildContentWhere();
                	$orderby        = $this->_buildContentOrderBy();
                        $query="SELECT a.* FROM #__abbook as a"
                                . $where
                        . ' GROUP BY a.id '
                        . $orderby
                        ;
                        $this->_total = $this->_getListCount($query);
                }

                return $this->_total;
        }

        function getPagination()
        {
                // Lets load the content if it doesn't already exist
                if (empty($this->_pagination))
                {
                        jimport('joomla.html.pagination');
                        $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
                }

                return $this->_pagination;
        }

        function _buildContentOrderBy()
        {
                global $mainframe, $option;

                $filter_order = $mainframe->getUserStateFromRequest( $option.'.books.filter_order', 'filter_order', 'a.ordering', 'cmd' );
                $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'.books.filter_order_Dir', 'filter_order_Dir', '', 'word' );
		// sanitize $filter_order
		if (!in_array($filter_order, array('a.title', 'a.catalogo', 'a.ordering', 'a.date', 'a.hits', 'rating', 'groupname', 'a.dateinsert'))) {
                        $filter_order = 'a.ordering';
                }	
		if ($filter_order == 'a.ordering'){
			$orderby = ' ORDER BY catid, '.$filter_order .' '.$filter_order_Dir;
		}else{
			$orderby = ' ORDER BY '.$filter_order .' '.$filter_order_Dir;
		}

                return $orderby;
        }

        function _buildContentWhere()
        {
                global $mainframe, $option;
		
		$db                                     =& JFactory::getDBO();
                $filter_state=$mainframe->getUserStateFromRequest($option.'.books.filter_state', 'filter_state', '', 'word');
		$filter_catid=$mainframe->getUserStateFromRequest( $option.'.books.filter_catid', 'filter_catid', 0, 'int' );
                $search            = $mainframe->getUserStateFromRequest( $option.'.books.search', 'search', '', 'string' );
                $search                 = JString::strtolower( $search );
                $where = array();

                if ( $filter_state ) {
                        if ( $filter_state == 'P' ) {
                                $where[] = 'a.published = 1';
                        } else if ($filter_state == 'U' ) {
                                $where[] = 'a.published = 0';
                        }
                }
		
		if ($filter_catid > 0) {
                        $where[] = 'a.catid = '.(int) $filter_catid;
                }

		if ($search) {
                        $where[] = 'LOWER(a.title) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
                }

                $where          = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );

                return $where;
        }

        function publish($cid = array(), $publish = 1)
        {
                $user   =& JFactory::getUser();
                $userid = (int) $user->get('id');

                if (count( $cid ))
                {
                        $cids = implode( ',', $cid );

                        $query = 'UPDATE #__abbook'
                                . ' SET published = '. (int) $publish
                                . ' WHERE id IN ('. $cids .')'
                        ;
                        $this->_db->setQuery( $query );

                        if (!$this->_db->query()) {
                                $this->setError($this->_db->getErrorMsg());
                                return false;
                        }
                }
        }

	function &getCategorieslist() {
                if (empty( $this->_categorieslist )) {
                        $query = ' SELECT *'
                        . ' FROM #__abcategories'
			. ' ORDER BY parent_id, ordering'
                        ;
                }
                if (empty($this->_categorieslist)) {
                        $this->_db->setQuery( $query );
                        $rows = $this->_getList( $query );
                }
                $children = array();
                        // first pass - collect children
                        foreach ($rows as $v ){
                                $pt = $v->parent_id;
                                $list = @$children[$pt] ? $children[$pt] : array();
                                array_push( $list, $v );
                                $children[$pt] = $list;
                        }
                        // second pass - get an indent list of the items
			JHTML::addIncludePath(JPATH_COMPONENT_SITE.DS.'helpers');
			$this->_categorieslist = JHTML::_('menu.treerecurse', 0, '', '', array(), $children);
                return $this->_categorieslist;
        }
	
	function move($direction)
	{
                $row =& JTable::getInstance('book', 'Table');
                if (!$row->load( $this->_id ) ) {
                        $this->setError($this->_db->getErrorMsg());
                        return false;
                }

                if (!$row->move( $direction, 'catid= '.$row->catid )) {
                        $this->setError($this->_db->getErrorMsg());
                        return false;
                }

                return true;
        }

	function saveorder($cid = array(), $order)
        {
                $row =& JTable::getInstance('book', 'Table');

                // update ordering values
                for( $i=0; $i < count($cid); $i++ )
                {
                        $row->load( (int) $cid[$i] );

                        if ($row->ordering != $order[$i])
                        {
                                $row->ordering = $order[$i];
                                if (!$row->store()) {
                                        $this->setError($this->_db->getErrorMsg());
                                        return false;
                                }
                        }
                }

                return true;
        }
}
