<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010 Ugolotti Federica
 * @license GNU/GPL, see LICENSE.php
 * Abook is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class AbookModelCategories extends JModel
{
	var $_data;
	var $_total = null;
	var $_pagination = null;
        var $_id = null;

        function __construct()
        {
                parent::__construct();

                global $mainframe, $option;
		
		$params = &JComponentHelper::getParams( 'com_abook' );
		$pag=$params->get( 'categorypagination' );
                // Get the pagination request variables
                $limit          = $mainframe->getUserStateFromRequest( $option.'.categories.limit', 'limit', $pag, 'int' );
                $limitstart     = JRequest::getVar( 'limitstart', 0, 'int' );

                // In case limit has been changed, adjust limitstart accordingly
                $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

                $this->setState('limit', $limit);
                $this->setState('limitstart', $limitstart);
                $array = JRequest::getVar('cid',  0, '', 'array');
                $this->setId((int)$array[0]);
        }

        function setId($id)
        {
                // Set id and wipe data
                $this->_id       = $id;
                $this->_data = null;
        }

	function _buildQuery()
	{
                $where          = $this->_buildContentWhere();
                $orderby        = $this->_buildContentOrderBy();
		$user           =& JFactory::getUser();

		$query = "SELECT  c.*, c.parent_id, COUNT(CASE WHEN b.published=1 AND b.access <=".(int) $user->gid ." THEN b.id END) AS numero, "
			. "\n CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(':', c.id, c.alias) ELSE c.id END as slug "
			. "\n FROM (#__abcategories AS c"
			. "\n LEFT JOIN #__users AS u ON u.id = c.checked_out"
			. "\n LEFT JOIN #__groups AS g ON g.id = c.access)"
			. "\n LEFT JOIN #__abbook as b ON b.catid=c.id"
			. $where
			. "\n GROUP BY c.id"
			. $orderby
			;
		return $query;
	}


        function _buildContentOrderBy()
        {
		$config =& JComponentHelper::getParams( 'com_abook' );
                $order=$config->get( 'cat_display_order', 'ordering' );
		$order_dir=$config->get( 'display_order_dir', 'ASC' );
		$orderby        = ' ORDER BY c.parent_id, c.'.$order.' '.$order_dir;
                return $orderby;
        }

        function _buildContentWhere()
        {
                global $mainframe, $option;
		$user           =& JFactory::getUser();
                $db                                     =& JFactory::getDBO();
                $filter_order           = $mainframe->getUserStateFromRequest( $option.'.categories.filter_order', 'filter_order', 'name', 'cmd' );
                $filter_order_Dir       = $mainframe->getUserStateFromRequest( $option.'.categories.filter_order_Dir', 'filter_order_Dir', '', 'word' );
                $search                 = $mainframe->getUserStateFromRequest( $option.'.categories.search', 'search', '', 'string' );
                $search                         = JString::strtolower( $search );

                $where = array();

                if ($search) {
                        $where[] = 'LOWER(name) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
                }
		
		$where[] = "c.published = 1";
		$where[] = "c.access <=".(int) $user->gid;

                $where          = ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

                return $where;
        }

        function getData()
        {
                // Lets load the data if it doesn't already exist
                if (empty( $this->_data ))
                {
                        $db =& $this->getDBO();
                        $query = $this->_buildQuery();
                        $db->setQuery( $query );
                        $rows = $db->loadObjectList();
                        // establish the hierarchy of the menu
                        $children = array();
                        // first pass - collect children
                        foreach ($rows as $v ){
                                $pt = $v->parent_id;
                                $list = @$children[$pt] ? $children[$pt] : array();
                                array_push( $list, $v );
                                $children[$pt] = $list;
                        }
                        //$config =& JComponentHelper::getParams( 'com_abook' );
                        //$img=$config->get( 'catimage', 'folder_yellow.png' );
                        // second pass - get an indent list of the items
                        JHTML::addIncludePath(JPATH_COMPONENT.DS.'helpers');
                        $list = JHTML::_('menu.treerecurse',  0, '', true, array(), $children);
                        $this->_data = array_slice( $list, $this->getState('limitstart'), $this->getState('limit') );
                }
                return $this->_data;
        }

        function getTotal()
        {
                // Lets load the content if it doesn't already exist
                if (empty($this->_total))
                {
                        $query = $this->_buildQuery();
                        $this->_total = $this->_getListCount($query);
                }

                return $this->_total;
        }

        function getPagination()
        {
                // Lets load the content if it doesn't already exist
                if (empty($this->_pagination))
                {
                        jimport('joomla.html.pagination');
                        $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
                }

                return $this->_pagination;
        }

}
