<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010 Ugolotti Federica
 * @license GNU/GPL, see LICENSE.php
 * Abook is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class AbookModelCategory extends JModel
{
	var $_data;
	var $_total = null;
	var $_pagination = null;
        var $_id = null;

        function __construct()
        {
                parent::__construct();

                global $mainframe, $option;
		$params = &JComponentHelper::getParams( 'com_abook' );
		$pag=$params->get( 'bookpagination' );
                $limit          = $mainframe->getUserStateFromRequest( $option.'.limit', 'limit', $pag, 'int' );
                $limitstart     = JRequest::getVar( 'limitstart', 0, '', 'int' );
                // In case limit has been changed, adjust limitstart accordingly
                $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

                $this->setState('limit', $limit);
                $this->setState('limitstart', $limitstart);
                $array = JRequest::getVar('id',  0, '', 'array');
                $this->setId((int)$array[0]);
        }

        function setId($id)
        {
                // Set id and wipe data
                $this->_id       = $id;
                $this->_data = null;
        }
	
	/**
        ** books of this category
        **/
	function _buildQuery()
	{
                $where          = $this->_buildContentWhere();
                $orderby        = $this->_buildContentOrderBy();
		
		$query = "SELECT a.*, "
		. ' CASE WHEN CHAR_LENGTH(a.alias) THEN CONCAT_WS(\':\', a.id, a.alias) ELSE a.id END as slug, '
		. ' ROUND( v.rating_sum / v.rating_count ) AS rating, v.rating_count '
		. "\n FROM #__abbook AS a"
		. ' LEFT JOIN #__abrating AS v ON a.id = v.book_id'
		. $where
		. ' GROUP BY a.id '
		. $orderby
		;
		return $query;
	}

        function _buildContentOrderBy()
        {
                global $mainframe, $option;
		$config =& JComponentHelper::getParams( 'com_abook' );
                $order=$config->get( 'display_order' );
		$order_dir=$config->get( 'display_order_dir', "ASC" );
		$orderby        = ' ORDER BY '.$order.' '.$order_dir;
                return $orderby;
        }

        function _buildContentWhere()
        {
                global $mainframe, $option;
		$user           =& JFactory::getUser();
                $db                                     =& JFactory::getDBO();
                $filter_order           = $mainframe->getUserStateFromRequest( $option.'.categories.filter_order', 'filter_order', 'name', 'cmd' );
                $filter_order_Dir       = $mainframe->getUserStateFromRequest( $option.'.categories.filter_order_Dir', 'filter_order_Dir', '', 'word' );
                $search                 = $mainframe->getUserStateFromRequest( $option.'.categories.search', 'search', '', 'string' );
                $search                         = JString::strtolower( $search );

                $where = array();

                if ($search) {
                        $where[] = 'LOWER(name) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
                }
		
		$where[] = "catid=" .$this->_id;
		$where[] = "published = 1";
		$where[] = "a.access <=".(int) $user->gid;

                $where          = ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

                return $where;
        }

	/**
	** all categories
	**/
	function _buildCategoryQuery()
        {
                //$orderby        = $this->_buildContentOrderBy();
		$config =& JComponentHelper::getParams( 'com_abook' );
                $order=$config->get( 'cat_display_order' );
                $orderby        = ' ORDER BY '.$order;
		//$orderby        = ' ORDER BY name';
		$user           =& JFactory::getUser();

		$query = "SELECT  c.*, c.parent_id as parent, COUNT(CASE WHEN b.published=1 AND b.access <=".(int) $user->gid ." THEN b.id END) AS numero, "
                        . "\n CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(':', c.id, c.alias) ELSE c.id END as slug "
                        . "\n FROM (#__abcategories AS c"
                        . "\n LEFT JOIN #__users AS u ON u.id = c.checked_out"
                        . "\n LEFT JOIN #__groups AS g ON g.id = c.access)"
                        . "\n LEFT JOIN #__abbook as b ON b.catid=c.id"
                        . "\n WHERE c.published = 1"
                        . "\n GROUP BY c.id"
                        . $orderby
                        ;
                return $query;
        }

	function getParentCategory()
	{
		$cat =& $this->getTable('category');
                $cat->load($this->_id);
		$parent_id=$cat->parent_id;
		$n=0;
		while ($parent_id!=0){
			$cat->load($parent_id);
			$category[$n]->id=$cat->id;
			$category[$n]->name=$cat->name;
			$parent_id=$cat->parent_id;
			$n=$n + 1;
		}
		$this->_parentcategory=array_reverse($category);
		return $this->_parentcategory;
	}

	function getSubCategory()
        {
                if (empty($this->_subcategory))
                {
			$query = $this->_buildCategoryQuery();
			$rows = $this->_getList($query);
			// establish the hierarchy of the menu
                        $children = array();
                        // first pass - collect children
                        foreach ($rows as $v ){
                                $pt = $v->parent_id;
                                $list = @$children[$pt] ? $children[$pt] : array();
                                array_push( $list, $v );
                                $children[$pt] = $list;
                        }
			//$config =& JComponentHelper::getParams( 'com_abook' );
                	//$img=$config->get( 'catimage', 'folder_yellow.png' );
			
                        // second pass - get an indent list of the items
			JHTML::addIncludePath(JPATH_COMPONENT.DS.'helpers');
                        $this->_subcategory = JHTML::_('menu.treerecurse',  $this->_id, '', true, array(), $children);
		}
                return $this->_subcategory;
	}
	
	function getData()
	{
		// Lets load the data if it doesn't already exist
		if (empty( $this->_data ))
		{
                       	$query = $this->_buildQuery();
                       	$this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
			$total = count($this->_data);
			for($i = 0; $i < $total; $i++)
                        {
                                $item =& $this->_data[$i];
                                $item->author = $this->getAuthors($item->id);
                        }	
                }
		return $this->_data;
	}

	function getAuthors($idbook)
        {
                $query = "SELECT idbook, GROUP_CONCAT(name ORDER BY name ASC SEPARATOR ', ') AS author "
                . "\n FROM #__abbookauth "
                . ' LEFT JOIN #__abauthor ON id=idauth '
                . ' WHERE  idbook='.$idbook
                . ' GROUP BY idbook '
                ;
                $this->_db->setQuery($query);
                $authors=$this->_db->loadObject();
                return $authors->author;
        }

        function getTotal()
        {
                // Lets load the content if it doesn't already exist
                if (empty($this->_total))
                {
                        $query = $this->_buildQuery();
                        $this->_total = $this->_getListCount($query);
                }

                return $this->_total;
        }

        function getPagination()
        {
                // Lets load the content if it doesn't already exist
                if (empty($this->_pagination))
                {
                        jimport('joomla.html.pagination');
                        $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
                }

                return $this->_pagination;
        }

	function getCategory()
        {
                if (empty($this->_category))
                {
                        // current category info
                        $query = 'SELECT c.*, ' .
                                ' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug '.
                                ' FROM #__abcategories AS c' .
                                ' WHERE c.id = '. (int) $this->_id
                                ;
                        $this->_db->setQuery($query);
                        $this->_category = $this->_db->loadObject();
                }
		return $this->_category;
        }

	function &getCatname()
        {
                // Load the data
                if (empty( $this->_catname )) {
                        $cat =& $this->getTable('category');
                        if (!$cat->load($this->_id)) {
                                $this->setError($this->_db->getErrorMsg());
                                return false;
                        }
                        $parent[0]->id=$cat->id;
                        $parent[0]->name=$cat->name;
                        $parent[0]->slug=$cat->id.':'.$cat->alias;
                        $parent[0]->published=$cat->published;
                        $parent[0]->access=$cat->access;
                        $k=1;
                        while ($cat->parent_id!=0) {
                                $cat->load($cat->parent_id);
                                $parent[$k]->id=$cat->id;
                                $parent[$k]->name=$cat->name;
                                $parent[$k]->slug=$cat->id.':'.$cat->alias;
                                $parent[$k]->published=$cat->published;
                                $parent[$k]->access=$cat->access;
                                $k=$k+1;
                        }
                }
                $this->_catname = $parent;
                return $this->_catname;
        }
}
